@@ -13,6 +13,7 @@
  */
 #include <config.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include <gdk/gdkx.h>
 #include <gtk/gtksignal.h>
@@ -26,6 +27,8 @@
 #include <bonobo/bonobo-property-bag-client.h>
 
 enum {
+	PLUG_CREATED,
+	DISCONNECTED,
 	SET_FRAME,
 	ACTIVATE,
 	LAST_SIGNAL
@@ -48,10 +51,25 @@ struct _BonoboControlPrivate {
 	GtkWidget           *plug;
 	GtkWidget           *widget;
 
+	guint                no_frame_timeout_id;
+
 	guint                active : 1;
 	guint                automerge : 1;
+	guint                sent_disconnect : 1;
 };
 
+static void
+bonobo_control_disconnected (BonoboControl *control)
+{
+	g_return_if_fail (control != NULL);
+	g_return_if_fail (control->priv != NULL);
+
+	if (!control->priv->sent_disconnect) {
+		control->priv->sent_disconnect = TRUE;
+		g_signal_emit (control, control_signals [DISCONNECTED], 0);
+	}
+}
+
 static void
 control_frame_connection_died_cb (gpointer connection,
 				  gpointer user_data)
@@ -60,6 +78,8 @@ control_frame_connection_died_cb (gpointer connection,
 
 	g_return_if_fail (control != NULL);
 
+	bonobo_control_disconnected (control);
+
 	dprintf ("The remote control frame died unexpectedly");
 	bonobo_object_unref (BONOBO_OBJECT (control));
 }
@@ -208,10 +228,11 @@ impl_Bonobo_Control_activate (PortableServer_Servant servant,
 	 * if our active state is not what we are changing it to, then
 	 * don't emit the signal
 	 */
-	if (control->priv->active != activated)
+	if (control->priv->active == old_activated)
 		return;
 
-	g_signal_emit (control, control_signals [ACTIVATE], 0, (gboolean) activated);
+	g_signal_emit (control, control_signals [ACTIVATE], 0, control->priv->active);
+	bonobo_control_activate_notify (control, control->priv->active, ev);
 }
 
 static void
@@ -226,6 +247,11 @@ bonobo_control_unset_control_frame (BonoboControl     *control,
 	} else
 		ev = opt_ev;
 
+	if (control->priv->no_frame_timeout_id != 0) {
+		g_source_remove (control->priv->no_frame_timeout_id);
+		control->priv->no_frame_timeout_id = 0;
+	}
+
 	if (control->priv->frame != CORBA_OBJECT_NIL) {
 		Bonobo_ControlFrame frame = control->priv->frame;
 
@@ -261,10 +287,11 @@ create_plug (BonoboControl *control)
 		gtk_container_add (GTK_CONTAINER (plug),
 				   control->priv->widget);
 
+	g_signal_emit (control, control_signals [PLUG_CREATED], 0);
+
 	g_object_unref (G_OBJECT (plug));
 }
 
-#ifdef HAVE_GTK_MULTIHEAD
 static int
 parse_cookie (const CORBA_char *cookie)
 {
@@ -291,11 +318,11 @@ parse_cookie (const CORBA_char *cookie)
 		case '=':
 			if (!ident || value)
 				goto parse_failed;
-			value = g_string_new ("");
+			value = g_string_new (NULL);
 			break;
 		default:
 			if (!ident)
-				ident = g_string_new ("");
+				ident = g_string_new (NULL);
 			
 			if (value)
 				g_string_append_c (value, *p);
@@ -317,7 +344,6 @@ parse_failed:
 
 	return retval;
 }
-#endif /* HAVE_GTK_MULTIHEAD */
 
 static CORBA_char *
 impl_Bonobo_Control_getWindowId (PortableServer_Servant servant,
@@ -327,17 +353,14 @@ impl_Bonobo_Control_getWindowId (PortableServer_Servant servant,
 	guint32        x11_id;
 	BonoboControl *control = BONOBO_CONTROL (
 		bonobo_object_from_servant (servant));
-#ifdef HAVE_GTK_MULTIHEAD
 	GdkScreen *gdkscreen;
 	int        screen_num;
-#endif
 
 	if (!control->priv->plug)
 		create_plug (control);
 
 	g_assert (control->priv->plug != NULL);
 
-#ifdef HAVE_GTK_MULTIHEAD
 	screen_num = parse_cookie (cookie);
 	if (screen_num != -1)
 		gdkscreen = gdk_display_get_screen (
@@ -346,7 +369,6 @@ impl_Bonobo_Control_getWindowId (PortableServer_Servant servant,
 		gdkscreen = gdk_screen_get_default ();
 
 	gtk_window_set_screen (GTK_WINDOW (control->priv->plug), gdkscreen);
-#endif
 
 	gtk_widget_show (control->priv->plug);
 
@@ -419,11 +441,8 @@ impl_Bonobo_Control_setSize (PortableServer_Servant  servant,
 	size.height = height;
 
 	/*
-	 * Nothing.
-	 *
-	 * In the Gnome implementation of Bonobo, all size negotiation
-	 * is handled by GtkPlug/GtkSocket for us, or GtkFrame in the
-	 * local case.
+	 * In the Gnome implementation of Bonobo, all size assignment
+	 * is handled by GtkPlug/GtkSocket for us.
 	 */
 
 	g_warning ("setSize untested");
@@ -556,6 +575,34 @@ impl_Bonobo_Control_focus (PortableServer_Servant servant,
 	return gtk_widget_child_focus (GTK_WIDGET (priv->plug), direction);
 }
 
+#define DEFAULT_CONTROL_PURGE_DELAY_MS 60 * 1000 /* 60 seconds */
+
+static int control_purge_delay = DEFAULT_CONTROL_PURGE_DELAY_MS;
+
+/**
+ * bonobo_control_life_set_purge:
+ * @ms: time to wait in milliseconds.
+ * 
+ *   Set time we're prepared to wait without a ControlFrame
+ * before terminating the Control. This can happen if the
+ * panel activates us but crashes before the set_frame.
+ **/
+void
+bonobo_control_life_set_purge (long ms)
+{
+	control_purge_delay = ms;
+}
+
+static gboolean
+never_got_frame_timeout (gpointer user_data)
+{
+	g_warning ("Never got frame, control died - abnormal exit condition");
+
+	bonobo_control_disconnected (user_data);
+	
+	return FALSE;
+}
+
 BonoboControl *
 bonobo_control_construct (BonoboControl  *control,
 			  GtkWidget      *widget)
@@ -570,6 +617,15 @@ bonobo_control_construct (BonoboControl  *control,
 	 */
 	bonobo_setup_x_error_handler ();
 
+	/*
+	 *   Start the clock ticking until we emit 'disconnected'
+	 * since, we never got a valid frame.
+	 */
+	control->priv->no_frame_timeout_id = g_timeout_add (
+		control_purge_delay,
+		(GSourceFunc) never_got_frame_timeout,
+		control);
+
 	control->priv->widget = g_object_ref (widget);
 	gtk_object_sink (GTK_OBJECT (widget));
 
@@ -671,6 +727,7 @@ bonobo_control_destroy (BonoboObject *object)
 	bonobo_control_unset_control_frame (control, NULL);
 	bonobo_control_set_properties      (control, CORBA_OBJECT_NIL, NULL);
 	bonobo_control_set_ui_component    (control, NULL);
+	bonobo_control_disconnected (control);
 
 	if (control->priv->widget) {
 		gtk_widget_destroy (GTK_WIDGET (control->priv->widget));
@@ -831,7 +888,7 @@ bonobo_control_get_ambient_properties (BonoboControl     *control,
 {
 	Bonobo_ControlFrame frame;
 	Bonobo_PropertyBag pbag;
-	CORBA_Environment *ev = 0, tmp_ev;
+	CORBA_Environment *ev = NULL, tmp_ev;
 
 	g_return_val_if_fail (BONOBO_IS_CONTROL (control), NULL);
 
@@ -941,6 +998,24 @@ bonobo_control_class_init (BonoboControlClass *klass)
 
 	bonobo_control_parent_class = g_type_class_peek_parent (klass);
 
+	control_signals [PLUG_CREATED] =
+                g_signal_new ("plug_created",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (BonoboControlClass, plug_created),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
+
+	control_signals [DISCONNECTED] =
+                g_signal_new ("disconnected",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (BonoboControlClass, disconnected),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE, 0);
+
 	control_signals [SET_FRAME] =
                 g_signal_new ("set_frame",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -990,7 +1065,7 @@ bonobo_control_init (BonoboControl *control)
 BONOBO_TYPE_FUNC_FULL (BonoboControl, 
 		       Bonobo_Control,
 		       BONOBO_OBJECT_TYPE,
-		       bonobo_control);
+		       bonobo_control)
 
 /*
  * Varrarg Property get/set simplification wrappers.
@@ -1178,9 +1253,10 @@ bonobo_control_set_transient_for (BonoboControl     *control,
 				  CORBA_Environment *opt_ev)
 {
 	CORBA_char         *id;
+	GdkDisplay         *display;
 	GdkWindow          *win;
 	guint32             x11_id;
-	CORBA_Environment  *ev = 0, tmp_ev;
+	CORBA_Environment  *ev = NULL, tmp_ev;
 	Bonobo_ControlFrame frame;
 
 	g_return_if_fail (GTK_IS_WINDOW (window));
@@ -1214,7 +1290,8 @@ bonobo_control_set_transient_for (BonoboControl     *control,
 	CORBA_free (id);
 
 	/* FIXME: Special case the local case ? */
-	win = gdk_window_foreign_new (x11_id);
+	display = gtk_widget_get_display (GTK_WIDGET (window));
+	win = gdk_window_foreign_new_for_display (display, x11_id);
 	g_return_if_fail (win != NULL);
 
 	window_set_transient_for_gdk (window, win);
@@ -1279,6 +1356,18 @@ bonobo_control_set_plug (BonoboControl *control,
 		bonobo_plug_set_control (plug, control);
 }
 
+/**
+ * bonobo_control_get_plug:
+ * @control: the control.a
+ * 
+ * This methods returns the current plug associated with
+ * the control. If the remote container re-parents the
+ * control - the plug will die, and a new plug has to be
+ * created. Thus this should really only be called from
+ * a 'plug_created' signal handler.
+ * 
+ * Return value: the _current_ plug.
+ **/
 BonoboPlug *
 bonobo_control_get_plug (BonoboControl *control)
 {
@@ -1356,15 +1445,15 @@ bonobo_control_get_popup_ui_component (BonoboControl *control)
 }
 
 gboolean
-bonobo_control_do_popup_full (BonoboControl       *control,
+bonobo_control_do_popup_path (BonoboControl       *control,
 			      GtkWidget           *parent_menu_shell,
 			      GtkWidget           *parent_menu_item,
 			      GtkMenuPositionFunc  func,
 			      gpointer             data,
 			      guint                button,
+			      const char          *popup_path,
 			      guint32              activate_time)
 {
-	char      *path;
 	GtkWidget *menu;
 
 	g_return_val_if_fail (BONOBO_IS_CONTROL (control), FALSE);
@@ -1372,21 +1461,15 @@ bonobo_control_do_popup_full (BonoboControl       *control,
 	if (!control->priv->popup_ui_container)
 		return FALSE;
 
-	path = g_strdup_printf ("/popups/button%d", button);
-
 	menu = gtk_menu_new ();
 
 	bonobo_ui_sync_menu_add_popup (
 		BONOBO_UI_SYNC_MENU (control->priv->popup_ui_sync),
-		GTK_MENU (menu), path);
+		GTK_MENU (menu), popup_path);
 
-	g_free (path);
-
-#ifdef HAVE_GTK_MULTIHEAD
 	gtk_menu_set_screen (
 		GTK_MENU (menu),
 		gtk_window_get_screen (GTK_WINDOW (control->priv->plug)));
-#endif /* HAVE_GTK_MULTIHEAD */
 
 	gtk_widget_show (menu);
 
@@ -1398,6 +1481,29 @@ bonobo_control_do_popup_full (BonoboControl       *control,
 	return TRUE;
 }
 
+gboolean
+bonobo_control_do_popup_full (BonoboControl       *control,
+			      GtkWidget           *parent_menu_shell,
+			      GtkWidget           *parent_menu_item,
+			      GtkMenuPositionFunc  func,
+			      gpointer             data,
+			      guint                button,
+			      guint32              activate_time)
+{
+	char    *path;
+	gboolean retval;
+
+	path = g_strdup_printf ("/popups/button%d", button);
+
+	retval = bonobo_control_do_popup_path
+		(control, parent_menu_shell, parent_menu_item,
+		 func, data, button, path, activate_time);
+
+	g_free (path);
+
+	return retval;
+}
+
 gboolean
 bonobo_control_do_popup (BonoboControl       *control,
 			 guint                button,
@@ -1407,3 +1513,73 @@ bonobo_control_do_popup (BonoboControl       *control,
 		control, NULL, NULL, NULL, NULL,
 		button, activate_time);
 }
+
+/* Track the living controls */
+static GSList *live_controls = NULL;
+static BonoboControlLifeCallback life_callback =
+	(BonoboControlLifeCallback) bonobo_main_quit;
+
+/**
+ * bonobo_control_life_set_callback:
+ * @all_dead_callback: method to call at idle when no controls remain
+ * 
+ * See #bonobo_control_life_instrument
+ **/
+void
+bonobo_control_life_set_callback (BonoboControlLifeCallback all_dead_callback)
+{
+	life_callback = all_dead_callback;
+}
+
+static gboolean
+control_life_idle_cb (gpointer data)
+{
+	if (!live_controls && life_callback)
+		life_callback ();
+
+	return FALSE;
+}
+
+static void
+control_life_disconnected (BonoboControl *control)
+{
+	live_controls = g_slist_remove (live_controls, control);
+
+	if (!live_controls)
+		g_idle_add (control_life_idle_cb, NULL);
+}
+
+/**
+ * bonobo_control_life_instrument:
+ * @control: control to manage.
+ * 
+ * Request that @control is lifecycle managed by this code;
+ * when it (and all other registerees are dead, the
+ * all_dead_callback set by #bonobo_control_life_set_callback
+ * will be called at idle.
+ **/
+void
+bonobo_control_life_instrument (BonoboControl *control)
+{
+	g_return_if_fail (BONOBO_IS_CONTROL (control));
+
+	g_signal_connect (control, "disconnected",
+			  G_CALLBACK (control_life_disconnected),
+			  NULL);
+
+	live_controls = g_slist_prepend (live_controls, control);
+}
+
+/**
+ * bonobo_control_life_get_count:
+ * @void: 
+ * 
+ * calculates the number of live controls under management.
+ * 
+ * Return value: the number of live controls.
+ **/
+int
+bonobo_control_life_get_count (void)
+{
+	return g_slist_length (live_controls);
+}
