@@ -32,6 +32,10 @@
  */
 
 #include <config.h>
+#include "bonobo-dock-item.h"
+#include "bonobo-dock-band.h"
+#include "bonobo-dock-item-grip.h"
+#include "bonobo-ui-marshal.h"
 
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
@@ -40,13 +44,8 @@
 #include <gtk/gtktoolbar.h>
 #include <gtk/gtkwindow.h>
 
+#include <glib/gi18n.h>
 #include <libgnome/gnome-macros.h>
-#include <bonobo/bonobo-i18n.h>
-#include <bonobo/bonobo-dock-band.h>
-#include <bonobo/bonobo-dock-item.h>
-#include <bonobo/bonobo-ui-private.h>
-#include <bonobo/bonobo-ui-marshal.h>
-#include <bonobo/bonobo-dock-item-grip.h>
 
 struct _BonoboDockItemPrivate {
 	GtkWidget *grip;
@@ -132,6 +131,8 @@ check_guint_arg (GObject *object,
 {
   GParamSpec *pspec;
 
+  g_return_val_if_fail (object != NULL, FALSE);
+
   pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (object), name);
   if (pspec != NULL) {
     GValue value = { 0, };
@@ -154,6 +155,9 @@ get_preferred_width (BonoboDockItem *dock_item)
 
   child = GTK_BIN (dock_item)->child;
 
+  if (!child)
+    return 0;
+
   if (! check_guint_arg (G_OBJECT (child), "preferred_width", &preferred_width))
     {
       GtkRequisition child_requisition;
@@ -178,6 +182,9 @@ get_preferred_height (BonoboDockItem *dock_item)
 
   child = GTK_BIN (dock_item)->child;
 
+  if (!child)
+    return 0;
+
   if (! check_guint_arg (G_OBJECT (child), "preferred_height", &preferred_height))
     {
       GtkRequisition child_requisition;
@@ -195,17 +202,15 @@ get_preferred_height (BonoboDockItem *dock_item)
 }
 
 static void
-bonobo_dock_item_class_init (BonoboDockItemClass *class)
+bonobo_dock_item_class_init (BonoboDockItemClass *klass)
 {
-  GtkObjectClass *object_class;
   GObjectClass *gobject_class;
   GtkWidgetClass *widget_class;
   GtkContainerClass *container_class;
 
-  object_class = (GtkObjectClass *) class;
-  gobject_class = (GObjectClass *) class;
-  widget_class = (GtkWidgetClass *) class;
-  container_class = (GtkContainerClass *) class;
+  gobject_class = (GObjectClass *) klass;
+  widget_class = (GtkWidgetClass *) klass;
+  container_class = (GtkContainerClass *) klass;
 
   gobject_class->set_property = bonobo_dock_item_set_property;
   gobject_class->get_property = bonobo_dock_item_get_property;
@@ -252,7 +257,7 @@ bonobo_dock_item_class_init (BonoboDockItemClass *class)
 
   dock_item_signals[DOCK_DRAG_BEGIN] =
 	  g_signal_new ("dock_drag_begin",
-			G_TYPE_FROM_CLASS (object_class),
+			G_TYPE_FROM_CLASS (gobject_class),
 			G_SIGNAL_RUN_LAST,
 			G_STRUCT_OFFSET (BonoboDockItemClass,
 					 dock_drag_begin),
@@ -262,7 +267,7 @@ bonobo_dock_item_class_init (BonoboDockItemClass *class)
 
   dock_item_signals[DOCK_DRAG_MOTION] =
 	  g_signal_new ("dock_drag_motion",
-			G_TYPE_FROM_CLASS (object_class),
+			G_TYPE_FROM_CLASS (gobject_class),
 			G_SIGNAL_RUN_LAST,
 			G_STRUCT_OFFSET (BonoboDockItemClass, dock_drag_motion),
 			NULL, NULL,
@@ -271,7 +276,7 @@ bonobo_dock_item_class_init (BonoboDockItemClass *class)
 
   dock_item_signals[DOCK_DRAG_END] =
 	  g_signal_new ("dock_drag_end",
-			G_TYPE_FROM_CLASS (object_class),
+			G_TYPE_FROM_CLASS (gobject_class),
 			G_SIGNAL_RUN_LAST,
 			G_STRUCT_OFFSET (BonoboDockItemClass, dock_drag_end),
 			NULL, NULL,
@@ -280,7 +285,7 @@ bonobo_dock_item_class_init (BonoboDockItemClass *class)
 
   dock_item_signals[DOCK_DETACH] =
 	  g_signal_new ("dock_detach",
-			G_TYPE_FROM_CLASS (object_class),
+			G_TYPE_FROM_CLASS (gobject_class),
 			G_SIGNAL_RUN_LAST,
 			G_STRUCT_OFFSET (BonoboDockItemClass, dock_detach),
 			NULL, NULL,
@@ -289,7 +294,7 @@ bonobo_dock_item_class_init (BonoboDockItemClass *class)
 
   dock_item_signals[ORIENTATION_CHANGED] =
 	  g_signal_new ("orientation_changed",
-			G_TYPE_FROM_CLASS (object_class),
+			G_TYPE_FROM_CLASS (gobject_class),
 			G_SIGNAL_RUN_LAST,
 			G_STRUCT_OFFSET (BonoboDockItemClass, orientation_changed),
 			NULL, NULL,
