@@ -20,9 +20,12 @@
 #define GLOBAL_INTERFACE_KEY "/desktop/gnome/interface"
 
 static GConfEnumStringPair toolbar_styles[] = {
-        { BONOBO_UI_TOOLBAR_STYLE_PRIORITY_TEXT, "text" },
-        { BONOBO_UI_TOOLBAR_STYLE_ICONS_ONLY, "icons" },
-        { BONOBO_UI_TOOLBAR_STYLE_ICONS_AND_TEXT, "both" }
+        { BONOBO_UI_TOOLBAR_STYLE_TEXT_ONLY,      "text" },
+        { BONOBO_UI_TOOLBAR_STYLE_ICONS_ONLY,     "icons" },
+        { BONOBO_UI_TOOLBAR_STYLE_ICONS_AND_TEXT, "both" },
+        { BONOBO_UI_TOOLBAR_STYLE_PRIORITY_TEXT,  "both_horiz" },
+	{ BONOBO_UI_TOOLBAR_STYLE_PRIORITY_TEXT,  "both-horiz" },
+	{ -1, NULL }
 };
 
 static GConfClient *client;
@@ -165,13 +168,11 @@ bonobo_ui_preferences_get_toolbar_style (void)
 				       NULL);
 	
 	if (str != NULL) {
-		gconf_string_to_enum(toolbar_styles,
-				     str,
-				     (gint*)&style);
-		g_free(str);
+		gconf_string_to_enum (toolbar_styles,
+				      str, (gint *)&style);
+		g_free (str);
 	}
 
-
 	return style;
 }
 
@@ -187,6 +188,7 @@ keys_changed_fn (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer
 	const char *key_name;
 
 	key_name = gconf_entry_get_key (entry);
+	g_return_if_fail (key_name != NULL);
 
 	/* FIXME: update the values instead */
 	if (!strcmp (key_name, GLOBAL_INTERFACE_KEY "/toolbar_detachable"))
@@ -201,5 +203,5 @@ keys_changed_fn (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer
 	if (update_engines_idle_id != 0)
 		return;
 
-	update_engines_idle_id = gtk_idle_add (update_engines_idle_callback, NULL);
+	update_engines_idle_id = g_idle_add (update_engines_idle_callback, NULL);
 }
