@@ -724,7 +724,11 @@ bonobo_ui_util_translate_ui (BonoboUINode *node)
 			a->id = g_quark_from_static_string (str + 1);
 
 			old = a->value;
-			a->value = xmlStrdup (_(a->value));
+#ifdef ENABLE_NLS
+			a->value = xmlStrdup (gettext(a->value));
+#else
+			a->value = xmlStrdup (a->value);
+#endif
 			xmlFree (old);
 		}
 	}
@@ -956,6 +960,8 @@ bonobo_ui_util_set_ui (BonoboUIComponent *component,
 			component, fname, app_datadir, app_name);
 
 		ui = bonobo_ui_node_to_string (node, TRUE);
+		if (!ui)
+			return;
 
 		bonobo_ui_node_free (node);
 		
