@@ -7,14 +7,20 @@
  *
  * Copyright 2000 Ximian, Inc.
  */
-#include "config.h"
+#include <config.h>
+#include <string.h>
+#include <gdk/gdkkeysyms.h>
+
 #include <bonobo/bonobo-dock-item.h>
 #include <bonobo/bonobo-dock.h>
 #include <bonobo/bonobo-window.h>
 #include <libbonobo.h>
+#include <libgnome/gnome-i18n.h>
+#include <libgnome/gnome-macros.h>
 
 #include <bonobo/bonobo-ui-preferences.h>
 #include <bonobo/bonobo-ui-engine.h>
+#include <bonobo/bonobo-ui-private.h>
 #include <bonobo/bonobo-ui-sync-menu.h>
 #include <bonobo/bonobo-ui-sync-keys.h>
 #include <bonobo/bonobo-ui-sync-status.h>
@@ -23,7 +29,8 @@
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 
-GtkWindowClass *bonobo_window_parent_class = NULL;
+GNOME_CLASS_BOILERPLATE (BonoboWindow, bonobo_window,
+			 GtkWindow, GTK_TYPE_WINDOW);
 
 struct _BonoboWindowPrivate {
 	BonoboUIEngine *engine;
@@ -43,11 +50,12 @@ struct _BonoboWindowPrivate {
 	char           *name;		/* Win name */
 	char           *prefix;		/* Win prefix */
 
-	GtkWidget      *main_vbox;
-
 	GtkBox         *status;
+};
 
-	GtkWidget      *client_area;
+enum {
+	PROP_0,
+	PROP_WIN_NAME
 };
 
 /**
@@ -101,9 +109,8 @@ bonobo_window_set_contents (BonoboWindow *win,
 {
 	g_return_if_fail (win != NULL);
 	g_return_if_fail (win->priv != NULL);
-	g_return_if_fail (win->priv->client_area != NULL);
 
-	gtk_container_add (GTK_CONTAINER (win->priv->client_area), contents);
+	bonobo_dock_set_client_area (win->priv->dock, contents);
 }
 
 /**
@@ -115,21 +122,11 @@ bonobo_window_set_contents (BonoboWindow *win,
 GtkWidget *
 bonobo_window_get_contents (BonoboWindow *win)
 {
-	GList     *children;
-	GtkWidget *widget;
-
 	g_return_val_if_fail (win != NULL, NULL);
 	g_return_val_if_fail (win->priv != NULL, NULL);
 	g_return_val_if_fail (win->priv->dock != NULL, NULL);
 
-	children = gtk_container_children (
-		GTK_CONTAINER (win->priv->client_area));
-
-	widget = children ? children->data : NULL;
-
-	g_list_free (children);
-
-	return widget;
+	return bonobo_dock_get_client_area (win->priv->dock);
 }
 
 static void
@@ -139,11 +136,11 @@ bonobo_window_dispose (GObject *object)
 	
 	if (win->priv->engine) {
 		bonobo_ui_engine_dispose (win->priv->engine);
-		g_object_unref (G_OBJECT (win->priv->engine));
+		g_object_unref (win->priv->engine);
 		win->priv->engine = NULL;
 	}
 
-	G_OBJECT_CLASS (bonobo_window_parent_class)->dispose (object);
+	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
@@ -157,7 +154,7 @@ bonobo_window_finalize (GObject *object)
 
 	win->priv = NULL;
 
-	G_OBJECT_CLASS (bonobo_window_parent_class)->finalize (object);
+	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 /**
@@ -177,6 +174,7 @@ bonobo_window_get_accel_group (BonoboWindow *win)
 static BonoboWindowPrivate *
 construct_priv (BonoboWindow *win)
 {
+	GtkWidget *main_vbox;
 	BonoboWindowPrivate *priv;
 	BonoboDockItemBehavior behavior;
 
@@ -184,9 +182,13 @@ construct_priv (BonoboWindow *win)
 
 	priv->engine = bonobo_ui_engine_new (G_OBJECT (win));
 
+	main_vbox = gtk_vbox_new (FALSE, 0);
+	gtk_container_add (GTK_CONTAINER (win), main_vbox);
+
 	priv->dock = BONOBO_DOCK (bonobo_dock_new ());
-	gtk_container_add (GTK_CONTAINER (win),
-			   GTK_WIDGET    (priv->dock));
+	gtk_box_pack_start (GTK_BOX (main_vbox),
+			    GTK_WIDGET (priv->dock),
+			    TRUE, TRUE, 0);
 
 	behavior = (BONOBO_DOCK_ITEM_BEH_EXCLUSIVE
 		    | BONOBO_DOCK_ITEM_BEH_NEVER_VERTICAL);
@@ -201,40 +203,16 @@ construct_priv (BonoboWindow *win)
 	bonobo_dock_add_item (priv->dock, priv->menu_item,
 			     BONOBO_DOCK_TOP, 0, 0, 0, TRUE);
 
-	/* 
-	 * To have menubar relief agree with the toolbar (and have the relief outside of
-	 * smaller handles), substitute the dock item's relief for the menubar's relief,
-	 * but don't change the size of the menubar in the process. 
-	 */
-#ifdef FIXME
-	gtk_menu_bar_set_shadow_type (GTK_MENU_BAR (priv->menu), GTK_SHADOW_NONE);
-#endif
-#if 0
-	if (bonobo_ui_preferences_get_menubar_relief ()) {
-		guint border_width;
-
-		gtk_container_set_border_width (GTK_CONTAINER (priv->menu_item), 2);
-		border_width = GTK_CONTAINER (priv->menu)->border_width;
-		if (border_width >= 2)
-			border_width -= 2;
-		gtk_container_set_border_width (GTK_CONTAINER (priv->menu), border_width);
-	} else
-#endif
-
-	priv->main_vbox = gtk_vbox_new (FALSE, 0);
-	bonobo_dock_set_client_area (priv->dock, priv->main_vbox);
-
-	priv->client_area = gtk_vbox_new (FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (priv->main_vbox), priv->client_area, TRUE, TRUE, 0);
-
 	priv->status = GTK_BOX (gtk_hbox_new (FALSE, 0));
-	gtk_box_pack_start (GTK_BOX (priv->main_vbox), GTK_WIDGET (priv->status), FALSE, FALSE, 0);
+	gtk_box_pack_end (GTK_BOX (main_vbox),
+			  GTK_WIDGET (priv->status),
+			  FALSE, FALSE, 0);
 
 	priv->accel_group = gtk_accel_group_new ();
 	gtk_window_add_accel_group (GTK_WINDOW (win),
 				    priv->accel_group);
 
-	gtk_widget_show_all (GTK_WIDGET (priv->dock));
+	gtk_widget_show_all (GTK_WIDGET (main_vbox));
 	gtk_widget_hide (GTK_WIDGET (priv->status));
 
 	priv->sync_menu = bonobo_ui_sync_menu_new (
@@ -268,10 +246,12 @@ construct_priv (BonoboWindow *win)
 static void
 bonobo_window_show_all (GtkWidget *widget)
 {
+	GtkWidget *client;
 	BonoboWindow *win = BONOBO_WINDOW (widget);
 
-	if (win->priv->client_area)
-		gtk_widget_show_all (win->priv->client_area);
+	if (win->priv->dock &&
+	    (client = bonobo_dock_get_client_area (win->priv->dock)))
+		gtk_widget_show_all (client);
 
 	gtk_widget_show (widget);
 }
@@ -282,39 +262,104 @@ bonobo_window_key_press_event (GtkWidget *widget,
 {
 	gboolean handled;
 	BonoboUISyncKeys *sync;
+	BonoboWindow *window = (BonoboWindow *) widget;
 
-	handled = GTK_WIDGET_CLASS (bonobo_window_parent_class)->key_press_event (widget, event);
+	handled = GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
 	if (handled)
 		return TRUE;
 
-	sync = BONOBO_UI_SYNC_KEYS (
-		BONOBO_WINDOW (widget)->priv->sync_keys);
+	sync = BONOBO_UI_SYNC_KEYS (window->priv->sync_keys);
 	if (sync)
 		return bonobo_ui_sync_keys_binding_handle (widget, event, sync);
 
 	return FALSE;
 }
 
+static gboolean
+bonobo_window_key_release_event (GtkWidget *widget,
+				 GdkEventKey *event)
+{
+	return GTK_WIDGET_CLASS (parent_class)->key_release_event (widget, event);
+}
+
+static void
+bonobo_window_set_property (GObject         *object,
+			guint            prop_id,
+			 const GValue    *value,
+			GParamSpec      *pspec)
+{
+	BonoboWindow *window;
+
+	window = BONOBO_WINDOW (object);
+
+	switch (prop_id)
+	{
+	case PROP_WIN_NAME:
+		bonobo_window_set_name (window, g_value_get_string (value));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+bonobo_window_get_property (GObject         *object,
+			guint            prop_id,
+			GValue          *value,
+			GParamSpec      *pspec)
+{
+	BonoboWindow *window;
+
+	window = BONOBO_WINDOW (object);
+
+	switch (prop_id)
+	{
+	case PROP_WIN_NAME:
+		g_value_set_string (value, window->priv->name);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
 static void
 bonobo_window_class_init (BonoboWindowClass *klass)
 {
 	GObjectClass *gobject_class = (GObjectClass *) klass;
 	GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
 
-	bonobo_window_parent_class =
-		gtk_type_class (gtk_window_get_type ());
-
 	gobject_class->dispose  = bonobo_window_dispose;
 	gobject_class->finalize = bonobo_window_finalize;
+	gobject_class->set_property = bonobo_window_set_property;
+	gobject_class->get_property = bonobo_window_get_property;
 
 	widget_class->show_all = bonobo_window_show_all;
 	widget_class->key_press_event = bonobo_window_key_press_event;
+	widget_class->key_release_event = bonobo_window_key_release_event;
+
+	/* Properties: */
+	g_object_class_install_property (gobject_class,
+		PROP_WIN_NAME,
+		g_param_spec_string ("win_name",
+		_("Name"),
+		_("Name of the window - used for configuration serialization."),
+		NULL,
+		G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 }
 
 static void
-bonobo_window_init (BonoboWindow *win)
+bonobo_window_instance_init (BonoboWindow *win)
 {
+	BonoboUIContainer *ui_container = NULL;
+	
 	win->priv = construct_priv (win);
+	
+	/* Create UIContainer: */
+	ui_container = bonobo_ui_container_new ();
+	bonobo_ui_container_set_engine (ui_container, win->priv->engine);
+	bonobo_object_unref (BONOBO_OBJECT (ui_container));
 }
 
 /**
@@ -401,9 +446,10 @@ bonobo_window_get_ui_container (BonoboWindow *win)
  * @iu_container: the UI container
  * @win_name: the window name
  * @title: the window's title for the title bar
- * 
- * Construct a new BonbooWindow
- * 
+ *
+ * Don't use this ever - use construct time properties instead.
+ * TODO: Remove this when we are allowed API changes.
+ *
  * Return value: a constructed window
  **/
 GtkWidget *
@@ -435,44 +481,12 @@ bonobo_window_construct (BonoboWindow      *win,
  * Return value: a new BonoboWindow
  **/
 GtkWidget *
-bonobo_window_new (const char   *win_name,
-		   const char   *title)
+bonobo_window_new (const char *win_name,
+		   const char *title)
 {
-	BonoboWindow      *win;
-	BonoboUIContainer *ui_container;
+	BonoboWindow *win = g_object_new (BONOBO_TYPE_WINDOW, "win_name", win_name, "title", title, NULL);
 
-	win = gtk_type_new (BONOBO_TYPE_WINDOW);
-
-	ui_container = bonobo_ui_container_new ();
-
-	return bonobo_window_construct (
-		win, ui_container, win_name, title);
+	return GTK_WIDGET (win);
 }
 
-/**
- * bonobo_window_get_type:
- *
- * Returns: The GtkType for the BonoboWindow class.
- */
-GtkType
-bonobo_window_get_type (void)
-{
-	static GtkType type = 0;
 
-	if (!type) {
-		GtkTypeInfo info = {
-			"BonoboWindow",
-			sizeof (BonoboWindow),
-			sizeof (BonoboWindowClass),
-			(GtkClassInitFunc) bonobo_window_class_init,
-			(GtkObjectInitFunc) bonobo_window_init,
-			NULL, /* reserved 1 */
-			NULL, /* reserved 2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		type = gtk_type_unique (gtk_window_get_type (), &info);
-	}
-
-	return type;
-}
